@@ -22,18 +22,27 @@ func NewCommitCommands(gitCommon *GitCommon) *CommitCommands {
 
 // ResetAuthor resets the author of the topmost commit
 func (self *CommitCommands) ResetAuthor() error {
-	return self.cmd.New("git commit --allow-empty --only --no-edit --amend --reset-author").Run()
+	cmdArgs := NewGitCmd("commit").
+		Arg("--allow-empty", "--only", "--no-edit", "--amend", "--reset-author").
+		ToArgv()
+
+	return self.cmd.New(cmdArgs).Run()
 }
 
 // Sets the commit's author to the supplied value. Value is expected to be of the form 'Name <Email>'
 func (self *CommitCommands) SetAuthor(value string) error {
-	commandStr := fmt.Sprintf("git commit --allow-empty --only --no-edit --amend --author=%s", self.cmd.Quote(value))
-	return self.cmd.New(commandStr).Run()
+	cmdArgs := NewGitCmd("commit").
+		Arg("--allow-empty", "--only", "--no-edit", "--amend", "--author="+value).
+		ToArgv()
+
+	return self.cmd.New(cmdArgs).Run()
 }
 
 // ResetToCommit reset to commit
 func (self *CommitCommands) ResetToCommit(sha string, strength string, envVars []string) error {
-	return self.cmd.New(fmt.Sprintf("git reset --%s %s", strength, sha)).
+	cmdArgs := NewGitCmd("reset").Arg("--"+strength, sha).ToArgv()
+
+	return self.cmd.New(cmdArgs).
 		// prevents git from prompting us for input which would freeze the program
 		// TODO: see if this is actually needed here
 		AddEnvVars("GIT_TERMINAL_PROMPT=0").
@@ -45,70 +54,82 @@ func (self *CommitCommands) CommitCmdObj(message string) oscommands.ICmdObj {
 	messageArgs := self.commitMessageArgs(message)
 
 	skipHookPrefix := self.UserConfig.Git.SkipHookPrefix
-	noVerifyFlag := ""
-	if skipHookPrefix != "" && strings.HasPrefix(message, skipHookPrefix) {
-		noVerifyFlag = " --no-verify"
-	}
 
-	return self.cmd.New(fmt.Sprintf("git commit%s%s%s", noVerifyFlag, self.signoffFlag(), messageArgs))
+	cmdArgs := NewGitCmd("commit").
+		ArgIf(skipHookPrefix != "" && strings.HasPrefix(message, skipHookPrefix), "--no-verify").
+		ArgIf(self.signoffFlag() != "", self.signoffFlag()).
+		Arg(messageArgs...).
+		ToArgv()
+
+	return self.cmd.New(cmdArgs)
+}
+
+func (self *CommitCommands) RewordLastCommitInEditorCmdObj() oscommands.ICmdObj {
+	return self.cmd.New(NewGitCmd("commit").Arg("--allow-empty", "--amend", "--only").ToArgv())
 }
 
 // RewordLastCommit rewords the topmost commit with the given message
 func (self *CommitCommands) RewordLastCommit(message string) error {
 	messageArgs := self.commitMessageArgs(message)
-	return self.cmd.New(fmt.Sprintf("git commit --allow-empty --amend --only%s", messageArgs)).Run()
+
+	cmdArgs := NewGitCmd("commit").
+		Arg("--allow-empty", "--amend", "--only").
+		Arg(messageArgs...).
+		ToArgv()
+
+	return self.cmd.New(cmdArgs).Run()
 }
 
-func (self *CommitCommands) commitMessageArgs(message string) string {
+func (self *CommitCommands) commitMessageArgs(message string) []string {
 	msg, description, _ := strings.Cut(message, "\n")
-	descriptionArgs := ""
+	args := []string{"-m", msg}
+
 	if description != "" {
-		descriptionArgs = fmt.Sprintf(" -m %s", self.cmd.Quote(description))
+		args = append(args, "-m", description)
 	}
 
-	return fmt.Sprintf(" -m %s%s", self.cmd.Quote(msg), descriptionArgs)
+	return args
 }
 
 // runs git commit without the -m argument meaning it will invoke the user's editor
 func (self *CommitCommands) CommitEditorCmdObj() oscommands.ICmdObj {
-	return self.cmd.New(fmt.Sprintf("git commit%s%s", self.signoffFlag(), self.verboseFlag()))
+	cmdArgs := NewGitCmd("commit").
+		ArgIf(self.signoffFlag() != "", self.signoffFlag()).
+		ToArgv()
+
+	return self.cmd.New(cmdArgs)
 }
 
 func (self *CommitCommands) signoffFlag() string {
 	if self.UserConfig.Git.Commit.SignOff {
-		return " --signoff"
+		return "--signoff"
 	} else {
 		return ""
 	}
 }
 
-func (self *CommitCommands) verboseFlag() string {
-	switch self.config.UserConfig.Git.Commit.Verbose {
-	case "always":
-		return " --verbose"
-	case "never":
-		return " --no-verbose"
-	default:
-		return ""
-	}
-}
-
 // Get the subject of the HEAD commit
 func (self *CommitCommands) GetHeadCommitMessage() (string, error) {
-	message, err := self.cmd.New("git log -1 --pretty=%s").DontLog().RunWithOutput()
+	cmdArgs := NewGitCmd("log").Arg("-1", "--pretty=%s").ToArgv()
+
+	message, err := self.cmd.New(cmdArgs).DontLog().RunWithOutput()
 	return strings.TrimSpace(message), err
 }
 
 func (self *CommitCommands) GetCommitMessage(commitSha string) (string, error) {
-	cmdStr := "git rev-list --format=%B --max-count=1 " + commitSha
-	messageWithHeader, err := self.cmd.New(cmdStr).DontLog().RunWithOutput()
+	cmdArgs := NewGitCmd("rev-list").
+		Arg("--format=%B", "--max-count=1", commitSha).
+		ToArgv()
+
+	messageWithHeader, err := self.cmd.New(cmdArgs).DontLog().RunWithOutput()
 	message := strings.Join(strings.SplitAfter(messageWithHeader, "\n")[1:], "")
 	return strings.TrimSpace(message), err
 }
 
 func (self *CommitCommands) GetCommitDiff(commitSha string) (string, error) {
-	cmdStr := "git show --no-color " + commitSha
-	diff, err := self.cmd.New(cmdStr).DontLog().RunWithOutput()
+	cmdArgs := NewGitCmd("show").Arg("--no-color", commitSha).ToArgv()
+
+	diff, err := self.cmd.New(cmdArgs).DontLog().RunWithOutput()
 	return diff, err
 }
 
@@ -118,8 +139,11 @@ type Author struct {
 }
 
 func (self *CommitCommands) GetCommitAuthor(commitSha string) (Author, error) {
-	cmdStr := "git show --no-patch --pretty=format:'%an%x00%ae' " + commitSha
-	output, err := self.cmd.New(cmdStr).DontLog().RunWithOutput()
+	cmdArgs := NewGitCmd("show").
+		Arg("--no-patch", "--pretty=format:'%an%x00%ae'", commitSha).
+		ToArgv()
+
+	output, err := self.cmd.New(cmdArgs).DontLog().RunWithOutput()
 	if err != nil {
 		return Author{}, err
 	}
@@ -138,15 +162,21 @@ func (self *CommitCommands) GetCommitMessageFirstLine(sha string) (string, error
 }
 
 func (self *CommitCommands) GetCommitMessagesFirstLine(shas []string) (string, error) {
-	return self.cmd.New(
-		fmt.Sprintf("git show --no-patch --pretty=format:%%s %s", strings.Join(shas, " ")),
-	).DontLog().RunWithOutput()
+	cmdArgs := NewGitCmd("show").
+		Arg("--no-patch", "--pretty=format:%s").
+		Arg(shas...).
+		ToArgv()
+
+	return self.cmd.New(cmdArgs).DontLog().RunWithOutput()
 }
 
 func (self *CommitCommands) GetCommitsOneline(shas []string) (string, error) {
-	return self.cmd.New(
-		fmt.Sprintf("git show --no-patch --oneline %s", strings.Join(shas, " ")),
-	).DontLog().RunWithOutput()
+	cmdArgs := NewGitCmd("show").
+		Arg("--no-patch", "--oneline").
+		Arg(shas...).
+		ToArgv()
+
+	return self.cmd.New(cmdArgs).DontLog().RunWithOutput()
 }
 
 // AmendHead amends HEAD with whatever is staged in your working tree
@@ -155,42 +185,58 @@ func (self *CommitCommands) AmendHead() error {
 }
 
 func (self *CommitCommands) AmendHeadCmdObj() oscommands.ICmdObj {
-	return self.cmd.New("git commit --amend --no-edit --allow-empty")
+	cmdArgs := NewGitCmd("commit").
+		Arg("--amend", "--no-edit", "--allow-empty").
+		ToArgv()
+
+	return self.cmd.New(cmdArgs)
 }
 
 func (self *CommitCommands) ShowCmdObj(sha string, filterPath string, ignoreWhitespace bool) oscommands.ICmdObj {
 	contextSize := self.UserConfig.Git.DiffContextSize
-	filterPathArg := ""
-	if filterPath != "" {
-		filterPathArg = fmt.Sprintf(" -- %s", self.cmd.Quote(filterPath))
-	}
-	ignoreWhitespaceArg := ""
-	if ignoreWhitespace {
-		ignoreWhitespaceArg = " --ignore-all-space"
-	}
 
-	cmdStr := fmt.Sprintf("git show --submodule --color=%s --unified=%d --stat -p %s%s%s",
-		self.UserConfig.Git.Paging.ColorArg, contextSize, sha, ignoreWhitespaceArg, filterPathArg)
-	return self.cmd.New(cmdStr).DontLog()
+	cmdArgs := NewGitCmd("show").
+		Arg("--submodule").
+		Arg("--color="+self.UserConfig.Git.Paging.ColorArg).
+		Arg(fmt.Sprintf("--unified=%d", contextSize)).
+		Arg("--stat").
+		Arg("--decorate").
+		Arg("-p").
+		Arg(sha).
+		ArgIf(ignoreWhitespace, "--ignore-all-space").
+		ArgIf(filterPath != "", "--", filterPath).
+		ToArgv()
+
+	return self.cmd.New(cmdArgs).DontLog()
 }
 
 // Revert reverts the selected commit by sha
 func (self *CommitCommands) Revert(sha string) error {
-	return self.cmd.New(fmt.Sprintf("git revert %s", sha)).Run()
+	cmdArgs := NewGitCmd("revert").Arg(sha).ToArgv()
+
+	return self.cmd.New(cmdArgs).Run()
 }
 
 func (self *CommitCommands) RevertMerge(sha string, parentNumber int) error {
-	return self.cmd.New(fmt.Sprintf("git revert %s -m %d", sha, parentNumber)).Run()
+	cmdArgs := NewGitCmd("revert").Arg(sha, "-m", fmt.Sprintf("%d", parentNumber)).
+		ToArgv()
+
+	return self.cmd.New(cmdArgs).Run()
 }
 
 // CreateFixupCommit creates a commit that fixes up a previous commit
 func (self *CommitCommands) CreateFixupCommit(sha string) error {
-	return self.cmd.New(fmt.Sprintf("git commit --fixup=%s", sha)).Run()
+	cmdArgs := NewGitCmd("commit").Arg("--fixup=" + sha).ToArgv()
+
+	return self.cmd.New(cmdArgs).Run()
 }
 
 // a value of 0 means the head commit, 1 is the parent commit, etc
 func (self *CommitCommands) GetCommitMessageFromHistory(value int) (string, error) {
-	hash, _ := self.cmd.New(fmt.Sprintf("git log -1 --skip=%d --pretty=%%H", value)).DontLog().RunWithOutput()
+	cmdArgs := NewGitCmd("log").Arg("-1", fmt.Sprintf("--skip=%d", value), "--pretty=%H").
+		ToArgv()
+
+	hash, _ := self.cmd.New(cmdArgs).DontLog().RunWithOutput()
 	formattedHash := strings.TrimSpace(hash)
 	if len(formattedHash) == 0 {
 		return "", ErrInvalidCommitIndex
