@@ -4,6 +4,7 @@ import (
 	"fmt"
 
 	"github.com/fsmiamoto/git-todo-parser/todo"
+	"github.com/jesseduffield/gocui"
 	"github.com/jesseduffield/lazygit/pkg/commands/models"
 	"github.com/jesseduffield/lazygit/pkg/commands/types/enums"
 	"github.com/jesseduffield/lazygit/pkg/gui/context"
@@ -13,26 +14,30 @@ import (
 	"github.com/samber/lo"
 )
 
+// after selecting the 200th commit, we'll load in all the rest
+const COMMIT_THRESHOLD = 200
+
 type (
 	PullFilesFn func() error
 )
 
 type LocalCommitsController struct {
 	baseController
-	*controllerCommon
+	c *ControllerCommon
+
 	pullFiles PullFilesFn
 }
 
 var _ types.IController = &LocalCommitsController{}
 
 func NewLocalCommitsController(
-	common *controllerCommon,
+	common *ControllerCommon,
 	pullFiles PullFilesFn,
 ) *LocalCommitsController {
 	return &LocalCommitsController{
-		baseController:   baseController{},
-		controllerCommon: common,
-		pullFiles:        pullFiles,
+		baseController: baseController{},
+		c:              common,
+		pullFiles:      pullFiles,
 	}
 }
 
@@ -41,75 +46,75 @@ func (self *LocalCommitsController) GetKeybindings(opts types.KeybindingsOpts) [
 		{
 			Key:         opts.GetKey(opts.Config.Commits.SquashDown),
 			Handler:     self.checkSelected(self.squashDown),
-			Description: self.c.Tr.LcSquashDown,
+			Description: self.c.Tr.SquashDown,
 		},
 		{
 			Key:         opts.GetKey(opts.Config.Commits.MarkCommitAsFixup),
 			Handler:     self.checkSelected(self.fixup),
-			Description: self.c.Tr.LcFixupCommit,
+			Description: self.c.Tr.FixupCommit,
 		},
 		{
 			Key:         opts.GetKey(opts.Config.Commits.RenameCommit),
 			Handler:     self.checkSelected(self.reword),
-			Description: self.c.Tr.LcRewordCommit,
+			Description: self.c.Tr.RewordCommit,
 		},
 		{
 			Key:         opts.GetKey(opts.Config.Commits.RenameCommitWithEditor),
 			Handler:     self.checkSelected(self.rewordEditor),
-			Description: self.c.Tr.LcRenameCommitEditor,
+			Description: self.c.Tr.RenameCommitEditor,
 		},
 		{
 			Key:         opts.GetKey(opts.Config.Universal.Remove),
 			Handler:     self.checkSelected(self.drop),
-			Description: self.c.Tr.LcDeleteCommit,
+			Description: self.c.Tr.DeleteCommit,
 		},
 		{
 			Key:         opts.GetKey(opts.Config.Universal.Edit),
 			Handler:     self.checkSelected(self.edit),
-			Description: self.c.Tr.LcEditCommit,
+			Description: self.c.Tr.EditCommit,
 		},
 		{
 			Key:         opts.GetKey(opts.Config.Commits.PickCommit),
 			Handler:     self.checkSelected(self.pick),
-			Description: self.c.Tr.LcPickCommit,
+			Description: self.c.Tr.PickCommit,
 		},
 		{
 			Key:         opts.GetKey(opts.Config.Commits.CreateFixupCommit),
 			Handler:     self.checkSelected(self.createFixupCommit),
-			Description: self.c.Tr.LcCreateFixupCommit,
+			Description: self.c.Tr.CreateFixupCommitDescription,
 		},
 		{
 			Key:         opts.GetKey(opts.Config.Commits.SquashAboveCommits),
 			Handler:     self.checkSelected(self.squashAllAboveFixupCommits),
-			Description: self.c.Tr.LcSquashAboveCommits,
+			Description: self.c.Tr.SquashAboveCommits,
 		},
 		{
 			Key:         opts.GetKey(opts.Config.Commits.MoveDownCommit),
 			Handler:     self.checkSelected(self.moveDown),
-			Description: self.c.Tr.LcMoveDownCommit,
+			Description: self.c.Tr.MoveDownCommit,
 		},
 		{
 			Key:         opts.GetKey(opts.Config.Commits.MoveUpCommit),
 			Handler:     self.checkSelected(self.moveUp),
-			Description: self.c.Tr.LcMoveUpCommit,
+			Description: self.c.Tr.MoveUpCommit,
 		},
 		{
 			Key:         opts.GetKey(opts.Config.Commits.PasteCommits),
 			Handler:     opts.Guards.OutsideFilterMode(self.paste),
-			Description: self.c.Tr.LcPasteCommits,
+			Description: self.c.Tr.PasteCommits,
 		},
 		// overriding these navigation keybindings because we might need to load
 		// more commits on demand
 		{
 			Key:         opts.GetKey(opts.Config.Universal.StartSearch),
 			Handler:     self.openSearch,
-			Description: self.c.Tr.LcStartSearch,
+			Description: self.c.Tr.StartSearch,
 			Tag:         "navigation",
 		},
 		{
 			Key:         opts.GetKey(opts.Config.Universal.GotoBottom),
 			Handler:     self.gotoBottom,
-			Description: self.c.Tr.LcGotoBottom,
+			Description: self.c.Tr.GotoBottom,
 			Tag:         "navigation",
 		},
 	}
@@ -122,27 +127,28 @@ func (self *LocalCommitsController) GetKeybindings(opts types.KeybindingsOpts) [
 		{
 			Key:         opts.GetKey(opts.Config.Commits.AmendToCommit),
 			Handler:     self.checkSelected(self.amendTo),
-			Description: self.c.Tr.LcAmendToCommit,
+			Description: self.c.Tr.AmendToCommit,
 		},
 		{
 			Key:         opts.GetKey(opts.Config.Commits.ResetCommitAuthor),
 			Handler:     self.checkSelected(self.amendAttribute),
-			Description: self.c.Tr.LcResetCommitAuthor,
+			Description: self.c.Tr.SetResetCommitAuthor,
+			OpensMenu:   true,
 		},
 		{
 			Key:         opts.GetKey(opts.Config.Commits.RevertCommit),
 			Handler:     self.checkSelected(self.revert),
-			Description: self.c.Tr.LcRevertCommit,
+			Description: self.c.Tr.RevertCommit,
 		},
 		{
 			Key:         opts.GetKey(opts.Config.Commits.CreateTag),
 			Handler:     self.checkSelected(self.createTag),
-			Description: self.c.Tr.LcTagCommit,
+			Description: self.c.Tr.TagCommit,
 		},
 		{
 			Key:         opts.GetKey(opts.Config.Commits.OpenLogMenu),
 			Handler:     self.handleOpenLogMenu,
-			Description: self.c.Tr.LcOpenLogMenu,
+			Description: self.c.Tr.OpenLogMenu,
 			OpensMenu:   true,
 		},
 	}...)
@@ -150,8 +156,53 @@ func (self *LocalCommitsController) GetKeybindings(opts types.KeybindingsOpts) [
 	return bindings
 }
 
+func (self *LocalCommitsController) GetOnRenderToMain() func() error {
+	return func() error {
+		return self.c.Helpers().Diff.WithDiffModeCheck(func() error {
+			var task types.UpdateTask
+			commit := self.context().GetSelected()
+			if commit == nil {
+				task = types.NewRenderStringTask(self.c.Tr.NoCommitsThisBranch)
+			} else if commit.Action == todo.UpdateRef {
+				task = types.NewRenderStringTask(
+					utils.ResolvePlaceholderString(
+						self.c.Tr.UpdateRefHere,
+						map[string]string{
+							"ref": commit.Name,
+						}))
+			} else {
+				cmdObj := self.c.Git().Commit.ShowCmdObj(commit.Sha, self.c.Modes().Filtering.GetPath(), self.c.State().GetIgnoreWhitespaceInDiffView())
+				task = types.NewRunPtyTask(cmdObj.GetCmd())
+			}
+
+			return self.c.RenderToMainViews(types.RefreshMainOpts{
+				Pair: self.c.MainViewPairs().Normal,
+				Main: &types.ViewUpdateOpts{
+					Title:    "Patch",
+					SubTitle: self.c.Helpers().Diff.IgnoringWhitespaceSubTitle(),
+					Task:     task,
+				},
+				Secondary: secondaryPatchPanelUpdateOpts(self.c),
+			})
+		})
+	}
+}
+
+func secondaryPatchPanelUpdateOpts(c *ControllerCommon) *types.ViewUpdateOpts {
+	if c.Git().Patch.PatchBuilder.Active() {
+		patch := c.Git().Patch.PatchBuilder.RenderAggregatedPatch(false)
+
+		return &types.ViewUpdateOpts{
+			Task:  types.NewRenderStringWithoutScrollTask(patch),
+			Title: c.Tr.CustomPatch,
+		}
+	}
+
+	return nil
+}
+
 func (self *LocalCommitsController) squashDown(commit *models.Commit) error {
-	if self.context().GetSelectedLineIdx() >= len(self.model.Commits)-1 {
+	if self.context().GetSelectedLineIdx() >= len(self.c.Model().Commits)-1 {
 		return self.c.ErrorMsg(self.c.Tr.CannotSquashOrFixupFirstCommit)
 	}
 
@@ -167,7 +218,7 @@ func (self *LocalCommitsController) squashDown(commit *models.Commit) error {
 		Title:  self.c.Tr.Squash,
 		Prompt: self.c.Tr.SureSquashThisCommit,
 		HandleConfirm: func() error {
-			return self.c.WithWaitingStatus(self.c.Tr.SquashingStatus, func() error {
+			return self.c.WithWaitingStatus(self.c.Tr.SquashingStatus, func(gocui.Task) error {
 				self.c.LogAction(self.c.Tr.Actions.SquashCommitDown)
 				return self.interactiveRebase(todo.Squash)
 			})
@@ -176,7 +227,7 @@ func (self *LocalCommitsController) squashDown(commit *models.Commit) error {
 }
 
 func (self *LocalCommitsController) fixup(commit *models.Commit) error {
-	if self.context().GetSelectedLineIdx() >= len(self.model.Commits)-1 {
+	if self.context().GetSelectedLineIdx() >= len(self.c.Model().Commits)-1 {
 		return self.c.ErrorMsg(self.c.Tr.CannotSquashOrFixupFirstCommit)
 	}
 
@@ -192,7 +243,7 @@ func (self *LocalCommitsController) fixup(commit *models.Commit) error {
 		Title:  self.c.Tr.Fixup,
 		Prompt: self.c.Tr.SureFixupThisCommit,
 		HandleConfirm: func() error {
-			return self.c.WithWaitingStatus(self.c.Tr.FixingStatus, func() error {
+			return self.c.WithWaitingStatus(self.c.Tr.FixingStatus, func(gocui.Task) error {
 				self.c.LogAction(self.c.Tr.Actions.FixupCommit)
 				return self.interactiveRebase(todo.Fixup)
 			})
@@ -209,12 +260,12 @@ func (self *LocalCommitsController) reword(commit *models.Commit) error {
 		return nil
 	}
 
-	commitMessage, err := self.git.Commit.GetCommitMessage(commit.Sha)
+	commitMessage, err := self.c.Git().Commit.GetCommitMessage(commit.Sha)
 	if err != nil {
 		return self.c.Error(err)
 	}
 
-	return self.helpers.Commits.OpenCommitMessagePanel(
+	return self.c.Helpers().Commits.OpenCommitMessagePanel(
 		&helpers.OpenCommitMessagePanelOpts{
 			CommitIndex:     self.context().GetSelectedLineIdx(),
 			InitialMessage:  commitMessage,
@@ -226,12 +277,12 @@ func (self *LocalCommitsController) reword(commit *models.Commit) error {
 }
 
 func (self *LocalCommitsController) handleReword(message string) error {
-	err := self.git.Rebase.RewordCommit(self.model.Commits, self.contexts.LocalCommits.GetSelectedLineIdx(), message)
+	err := self.c.Git().Rebase.RewordCommit(self.c.Model().Commits, self.c.Contexts().LocalCommits.GetSelectedLineIdx(), message)
 	if err != nil {
 		return self.c.Error(err)
 	}
-	self.helpers.Commits.OnCommitSuccess()
-	_ = self.helpers.Commits.PopCommitMessageContexts()
+	self.c.Helpers().Commits.OnCommitSuccess()
+	_ = self.c.Helpers().Commits.PopCommitMessageContexts()
 	return self.c.Refresh(types.RefreshOptions{Mode: types.ASYNC})
 }
 
@@ -239,11 +290,11 @@ func (self *LocalCommitsController) doRewordEditor() error {
 	self.c.LogAction(self.c.Tr.Actions.RewordCommit)
 
 	if self.isHeadCommit() {
-		return self.c.RunSubprocessAndRefresh(self.os.Cmd.New("git commit --allow-empty --amend --only"))
+		return self.c.RunSubprocessAndRefresh(self.c.Git().Commit.RewordLastCommitInEditorCmdObj())
 	}
 
-	subProcess, err := self.git.Rebase.RewordCommitInEditor(
-		self.model.Commits, self.context().GetSelectedLineIdx(),
+	subProcess, err := self.c.Git().Rebase.RewordCommitInEditor(
+		self.c.Model().Commits, self.context().GetSelectedLineIdx(),
 	)
 	if err != nil {
 		return self.c.Error(err)
@@ -288,7 +339,7 @@ func (self *LocalCommitsController) drop(commit *models.Commit) error {
 		Title:  self.c.Tr.DeleteCommitTitle,
 		Prompt: self.c.Tr.DeleteCommitPrompt,
 		HandleConfirm: func() error {
-			return self.c.WithWaitingStatus(self.c.Tr.DeletingStatus, func() error {
+			return self.c.WithWaitingStatus(self.c.Tr.DeletingStatus, func(gocui.Task) error {
 				self.c.LogAction(self.c.Tr.Actions.DropCommit)
 				return self.interactiveRebase(todo.Drop)
 			})
@@ -305,10 +356,10 @@ func (self *LocalCommitsController) edit(commit *models.Commit) error {
 		return nil
 	}
 
-	return self.c.WithWaitingStatus(self.c.Tr.RebasingStatus, func() error {
+	return self.c.WithWaitingStatus(self.c.Tr.RebasingStatus, func(gocui.Task) error {
 		self.c.LogAction(self.c.Tr.Actions.EditCommit)
-		err := self.git.Rebase.EditRebase(commit.Sha)
-		return self.helpers.MergeAndRebase.CheckMergeOrRebase(err)
+		err := self.c.Git().Rebase.EditRebase(commit.Sha)
+		return self.c.Helpers().MergeAndRebase.CheckMergeOrRebase(err)
 	})
 }
 
@@ -327,16 +378,20 @@ func (self *LocalCommitsController) pick(commit *models.Commit) error {
 }
 
 func (self *LocalCommitsController) interactiveRebase(action todo.TodoCommand) error {
-	err := self.git.Rebase.InteractiveRebase(self.model.Commits, self.context().GetSelectedLineIdx(), action)
-	return self.helpers.MergeAndRebase.CheckMergeOrRebase(err)
+	err := self.c.Git().Rebase.InteractiveRebase(self.c.Model().Commits, self.context().GetSelectedLineIdx(), action)
+	return self.c.Helpers().MergeAndRebase.CheckMergeOrRebase(err)
 }
 
 // handleMidRebaseCommand sees if the selected commit is in fact a rebasing
 // commit meaning you are trying to edit the todo file rather than actually
 // begin a rebase. It then updates the todo file with that action
 func (self *LocalCommitsController) handleMidRebaseCommand(action todo.TodoCommand, commit *models.Commit) (bool, error) {
+	if commit.Action == models.ActionConflict {
+		return true, self.c.ErrorMsg(self.c.Tr.ChangingThisActionIsNotAllowed)
+	}
+
 	if !commit.IsTODO() {
-		if self.git.Status.WorkingTreeState() != enums.REBASE_MODE_NONE {
+		if self.c.Git().Status.WorkingTreeState() != enums.REBASE_MODE_NONE {
 			// If we are in a rebase, the only action that is allowed for
 			// non-todo commits is rewording the current head commit
 			if !(action == todo.Reword && self.isHeadCommit()) {
@@ -352,11 +407,11 @@ func (self *LocalCommitsController) handleMidRebaseCommand(action todo.TodoComma
 	// our input or we set a lazygit client as the EDITOR env variable and have it
 	// request us to edit the commit message when prompted.
 	if action == todo.Reword {
-		return true, self.c.ErrorMsg(self.c.Tr.LcRewordNotSupported)
+		return true, self.c.ErrorMsg(self.c.Tr.RewordNotSupported)
 	}
 
 	if allowed := isChangeOfRebaseTodoAllowed(action); !allowed {
-		return true, self.c.ErrorMsg(self.c.Tr.LcChangingThisActionIsNotAllowed)
+		return true, self.c.ErrorMsg(self.c.Tr.ChangingThisActionIsNotAllowed)
 	}
 
 	self.c.LogAction("Update rebase TODO")
@@ -365,7 +420,7 @@ func (self *LocalCommitsController) handleMidRebaseCommand(action todo.TodoComma
 		false,
 	)
 
-	if err := self.git.Rebase.EditRebaseTodo(commit, action); err != nil {
+	if err := self.c.Git().Rebase.EditRebaseTodo(commit, action); err != nil {
 		return false, self.c.Error(err)
 	}
 
@@ -376,7 +431,7 @@ func (self *LocalCommitsController) handleMidRebaseCommand(action todo.TodoComma
 
 func (self *LocalCommitsController) moveDown(commit *models.Commit) error {
 	index := self.context().GetSelectedLineIdx()
-	commits := self.model.Commits
+	commits := self.c.Model().Commits
 
 	// can't move past the initial commit
 	if index >= len(commits)-1 {
@@ -384,7 +439,7 @@ func (self *LocalCommitsController) moveDown(commit *models.Commit) error {
 	}
 
 	if commit.IsTODO() {
-		if !commits[index+1].IsTODO() {
+		if !commits[index+1].IsTODO() || commits[index+1].Action == models.ActionConflict {
 			return nil
 		}
 
@@ -393,7 +448,7 @@ func (self *LocalCommitsController) moveDown(commit *models.Commit) error {
 		self.c.LogAction(self.c.Tr.Actions.MoveCommitDown)
 		self.c.LogCommand(fmt.Sprintf("Moving commit %s down", commit.ShortSha()), false)
 
-		if err := self.git.Rebase.MoveTodoDown(commit); err != nil {
+		if err := self.c.Git().Rebase.MoveTodoDown(commit); err != nil {
 			return self.c.Error(err)
 		}
 		self.context().MoveSelectedLine(1)
@@ -402,17 +457,17 @@ func (self *LocalCommitsController) moveDown(commit *models.Commit) error {
 		})
 	}
 
-	if self.git.Status.WorkingTreeState() != enums.REBASE_MODE_NONE {
+	if self.c.Git().Status.WorkingTreeState() != enums.REBASE_MODE_NONE {
 		return self.c.ErrorMsg(self.c.Tr.AlreadyRebasing)
 	}
 
-	return self.c.WithWaitingStatus(self.c.Tr.MovingStatus, func() error {
+	return self.c.WithWaitingStatus(self.c.Tr.MovingStatus, func(gocui.Task) error {
 		self.c.LogAction(self.c.Tr.Actions.MoveCommitDown)
-		err := self.git.Rebase.MoveCommitDown(self.model.Commits, index)
+		err := self.c.Git().Rebase.MoveCommitDown(self.c.Model().Commits, index)
 		if err == nil {
 			self.context().MoveSelectedLine(1)
 		}
-		return self.helpers.MergeAndRebase.CheckMergeOrRebase(err)
+		return self.c.Helpers().MergeAndRebase.CheckMergeOrRebase(err)
 	})
 }
 
@@ -431,7 +486,7 @@ func (self *LocalCommitsController) moveUp(commit *models.Commit) error {
 			false,
 		)
 
-		if err := self.git.Rebase.MoveTodoUp(self.model.Commits[index]); err != nil {
+		if err := self.c.Git().Rebase.MoveTodoUp(self.c.Model().Commits[index]); err != nil {
 			return self.c.Error(err)
 		}
 		self.context().MoveSelectedLine(-1)
@@ -440,29 +495,29 @@ func (self *LocalCommitsController) moveUp(commit *models.Commit) error {
 		})
 	}
 
-	if self.git.Status.WorkingTreeState() != enums.REBASE_MODE_NONE {
+	if self.c.Git().Status.WorkingTreeState() != enums.REBASE_MODE_NONE {
 		return self.c.ErrorMsg(self.c.Tr.AlreadyRebasing)
 	}
 
-	return self.c.WithWaitingStatus(self.c.Tr.MovingStatus, func() error {
+	return self.c.WithWaitingStatus(self.c.Tr.MovingStatus, func(gocui.Task) error {
 		self.c.LogAction(self.c.Tr.Actions.MoveCommitUp)
-		err := self.git.Rebase.MoveCommitUp(self.model.Commits, index)
+		err := self.c.Git().Rebase.MoveCommitUp(self.c.Model().Commits, index)
 		if err == nil {
 			self.context().MoveSelectedLine(-1)
 		}
-		return self.helpers.MergeAndRebase.CheckMergeOrRebase(err)
+		return self.c.Helpers().MergeAndRebase.CheckMergeOrRebase(err)
 	})
 }
 
 func (self *LocalCommitsController) amendTo(commit *models.Commit) error {
 	if self.isHeadCommit() {
-		if err := self.helpers.AmendHelper.AmendHead(); err != nil {
+		if err := self.c.Helpers().AmendHelper.AmendHead(); err != nil {
 			return err
 		}
 		return self.c.Refresh(types.RefreshOptions{Mode: types.ASYNC})
 	}
 
-	if self.git.Status.WorkingTreeState() != enums.REBASE_MODE_NONE {
+	if self.c.Git().Status.WorkingTreeState() != enums.REBASE_MODE_NONE {
 		return self.c.ErrorMsg(self.c.Tr.AlreadyRebasing)
 	}
 
@@ -470,17 +525,17 @@ func (self *LocalCommitsController) amendTo(commit *models.Commit) error {
 		Title:  self.c.Tr.AmendCommitTitle,
 		Prompt: self.c.Tr.AmendCommitPrompt,
 		HandleConfirm: func() error {
-			return self.c.WithWaitingStatus(self.c.Tr.AmendingStatus, func() error {
+			return self.c.WithWaitingStatus(self.c.Tr.AmendingStatus, func(gocui.Task) error {
 				self.c.LogAction(self.c.Tr.Actions.AmendCommit)
-				err := self.git.Rebase.AmendTo(self.model.Commits, self.context().GetView().SelectedLineIdx())
-				return self.helpers.MergeAndRebase.CheckMergeOrRebase(err)
+				err := self.c.Git().Rebase.AmendTo(self.c.Model().Commits, self.context().GetView().SelectedLineIdx())
+				return self.c.Helpers().MergeAndRebase.CheckMergeOrRebase(err)
 			})
 		},
 	})
 }
 
 func (self *LocalCommitsController) amendAttribute(commit *models.Commit) error {
-	if self.git.Status.WorkingTreeState() != enums.REBASE_MODE_NONE && !self.isHeadCommit() {
+	if self.c.Git().Status.WorkingTreeState() != enums.REBASE_MODE_NONE && !self.isHeadCommit() {
 		return self.c.ErrorMsg(self.c.Tr.AlreadyRebasing)
 	}
 
@@ -488,13 +543,13 @@ func (self *LocalCommitsController) amendAttribute(commit *models.Commit) error
 		Title: "Amend commit attribute",
 		Items: []*types.MenuItem{
 			{
-				Label:   "reset author",
+				Label:   self.c.Tr.ResetAuthor,
 				OnPress: self.resetAuthor,
 				Key:     'a',
 				Tooltip: "Reset the commit's author to the currently configured user. This will also renew the author timestamp",
 			},
 			{
-				Label:   "set author",
+				Label:   self.c.Tr.SetAuthor,
 				OnPress: self.setAuthor,
 				Key:     'A',
 				Tooltip: "Set the author based on a prompt",
@@ -504,9 +559,9 @@ func (self *LocalCommitsController) amendAttribute(commit *models.Commit) error
 }
 
 func (self *LocalCommitsController) resetAuthor() error {
-	return self.c.WithWaitingStatus(self.c.Tr.AmendingStatus, func() error {
+	return self.c.WithWaitingStatus(self.c.Tr.AmendingStatus, func(gocui.Task) error {
 		self.c.LogAction(self.c.Tr.Actions.ResetCommitAuthor)
-		if err := self.git.Rebase.ResetCommitAuthor(self.model.Commits, self.context().GetSelectedLineIdx()); err != nil {
+		if err := self.c.Git().Rebase.ResetCommitAuthor(self.c.Model().Commits, self.context().GetSelectedLineIdx()); err != nil {
 			return self.c.Error(err)
 		}
 
@@ -517,11 +572,11 @@ func (self *LocalCommitsController) resetAuthor() error {
 func (self *LocalCommitsController) setAuthor() error {
 	return self.c.Prompt(types.PromptOpts{
 		Title:               self.c.Tr.SetAuthorPromptTitle,
-		FindSuggestionsFunc: self.helpers.Suggestions.GetAuthorsSuggestionsFunc(),
+		FindSuggestionsFunc: self.c.Helpers().Suggestions.GetAuthorsSuggestionsFunc(),
 		HandleConfirm: func(value string) error {
-			return self.c.WithWaitingStatus(self.c.Tr.AmendingStatus, func() error {
+			return self.c.WithWaitingStatus(self.c.Tr.AmendingStatus, func(gocui.Task) error {
 				self.c.LogAction(self.c.Tr.Actions.SetCommitAuthor)
-				if err := self.git.Rebase.SetCommitAuthor(self.model.Commits, self.context().GetSelectedLineIdx(), value); err != nil {
+				if err := self.c.Git().Rebase.SetCommitAuthor(self.c.Model().Commits, self.context().GetSelectedLineIdx(), value); err != nil {
 					return self.c.Error(err)
 				}
 
@@ -544,7 +599,7 @@ func (self *LocalCommitsController) revert(commit *models.Commit) error {
 				}),
 			HandleConfirm: func() error {
 				self.c.LogAction(self.c.Tr.Actions.RevertCommit)
-				if err := self.git.Commit.Revert(commit.Sha); err != nil {
+				if err := self.c.Git().Commit.Revert(commit.Sha); err != nil {
 					return self.c.Error(err)
 				}
 				return self.afterRevertCommit()
@@ -557,7 +612,7 @@ func (self *LocalCommitsController) createRevertMergeCommitMenu(commit *models.C
 	menuItems := make([]*types.MenuItem, len(commit.Parents))
 	for i, parentSha := range commit.Parents {
 		i := i
-		message, err := self.git.Commit.GetCommitMessageFirstLine(parentSha)
+		message, err := self.c.Git().Commit.GetCommitMessageFirstLine(parentSha)
 		if err != nil {
 			return self.c.Error(err)
 		}
@@ -567,7 +622,7 @@ func (self *LocalCommitsController) createRevertMergeCommitMenu(commit *models.C
 			OnPress: func() error {
 				parentNumber := i + 1
 				self.c.LogAction(self.c.Tr.Actions.RevertCommit)
-				if err := self.git.Commit.RevertMerge(commit.Sha, parentNumber); err != nil {
+				if err := self.c.Git().Commit.RevertMerge(commit.Sha, parentNumber); err != nil {
 					return self.c.Error(err)
 				}
 				return self.afterRevertCommit()
@@ -598,7 +653,7 @@ func (self *LocalCommitsController) createFixupCommit(commit *models.Commit) err
 		Prompt: prompt,
 		HandleConfirm: func() error {
 			self.c.LogAction(self.c.Tr.Actions.CreateFixupCommit)
-			if err := self.git.Commit.CreateFixupCommit(commit.Sha); err != nil {
+			if err := self.c.Git().Commit.CreateFixupCommit(commit.Sha); err != nil {
 				return self.c.Error(err)
 			}
 
@@ -617,17 +672,17 @@ func (self *LocalCommitsController) squashAllAboveFixupCommits(commit *models.Co
 		Title:  self.c.Tr.SquashAboveCommits,
 		Prompt: prompt,
 		HandleConfirm: func() error {
-			return self.c.WithWaitingStatus(self.c.Tr.SquashingStatus, func() error {
+			return self.c.WithWaitingStatus(self.c.Tr.SquashingStatus, func(gocui.Task) error {
 				self.c.LogAction(self.c.Tr.Actions.SquashAllAboveFixupCommits)
-				err := self.git.Rebase.SquashAllAboveFixupCommits(commit)
-				return self.helpers.MergeAndRebase.CheckMergeOrRebase(err)
+				err := self.c.Git().Rebase.SquashAllAboveFixupCommits(commit)
+				return self.c.Helpers().MergeAndRebase.CheckMergeOrRebase(err)
 			})
 		},
 	})
 }
 
 func (self *LocalCommitsController) createTag(commit *models.Commit) error {
-	return self.helpers.Tags.CreateTagMenu(commit.Sha, func() {})
+	return self.c.Helpers().Tags.CreateTagMenu(commit.Sha, func() {})
 }
 
 func (self *LocalCommitsController) openSearch() error {
@@ -639,9 +694,7 @@ func (self *LocalCommitsController) openSearch() error {
 		}
 	}
 
-	self.c.OpenSearch()
-
-	return nil
+	return self.c.Helpers().Search.OpenSearchPrompt(self.context())
 }
 
 func (self *LocalCommitsController) gotoBottom() error {
@@ -671,7 +724,7 @@ func (self *LocalCommitsController) handleOpenLogMenu() error {
 						self.context().SetLimitCommits(false)
 					}
 
-					return self.c.WithWaitingStatus(self.c.Tr.LcLoadingCommits, func() error {
+					return self.c.WithWaitingStatus(self.c.Tr.LoadingCommits, func(gocui.Task) error {
 						return self.c.Refresh(
 							types.RefreshOptions{Mode: types.SYNC, Scope: []types.RefreshableView{types.COMMITS}},
 						)
@@ -714,7 +767,7 @@ func (self *LocalCommitsController) handleOpenLogMenu() error {
 					onPress := func(value string) func() error {
 						return func() error {
 							self.c.UserConfig.Git.Log.Order = value
-							return self.c.WithWaitingStatus(self.c.Tr.LcLoadingCommits, func() error {
+							return self.c.WithWaitingStatus(self.c.Tr.LoadingCommits, func(gocui.Task) error {
 								return self.c.Refresh(
 									types.RefreshOptions{
 										Mode:  types.SYNC,
@@ -759,20 +812,36 @@ func (self *LocalCommitsController) checkSelected(callback func(*models.Commit)
 	}
 }
 
+func (self *LocalCommitsController) GetOnFocus() func(types.OnFocusOpts) error {
+	return func(types.OnFocusOpts) error {
+		context := self.context()
+		if context.GetSelectedLineIdx() > COMMIT_THRESHOLD && context.GetLimitCommits() {
+			context.SetLimitCommits(false)
+			self.c.OnWorker(func(_ gocui.Task) {
+				if err := self.c.Refresh(types.RefreshOptions{Scope: []types.RefreshableView{types.COMMITS}}); err != nil {
+					_ = self.c.Error(err)
+				}
+			})
+		}
+
+		return nil
+	}
+}
+
 func (self *LocalCommitsController) Context() types.Context {
 	return self.context()
 }
 
 func (self *LocalCommitsController) context() *context.LocalCommitsContext {
-	return self.contexts.LocalCommits
+	return self.c.Contexts().LocalCommits
 }
 
 func (self *LocalCommitsController) paste() error {
-	return self.helpers.CherryPick.Paste()
+	return self.c.Helpers().CherryPick.Paste()
 }
 
 func (self *LocalCommitsController) isHeadCommit() bool {
-	return models.IsHeadCommit(self.model.Commits, self.context().GetSelectedLineIdx())
+	return models.IsHeadCommit(self.c.Model().Commits, self.context().GetSelectedLineIdx())
 }
 
 func isChangeOfRebaseTodoAllowed(action todo.TodoCommand) bool {
