@@ -1,6 +1,7 @@
 package gui
 
 import (
+	goContext "context"
 	"fmt"
 	"io"
 	"os"
@@ -8,6 +9,7 @@ import (
 	"sync"
 
 	"github.com/jesseduffield/gocui"
+	"github.com/jesseduffield/lazycore/pkg/boxlayout"
 	appTypes "github.com/jesseduffield/lazygit/pkg/app/types"
 	"github.com/jesseduffield/lazygit/pkg/commands"
 	"github.com/jesseduffield/lazygit/pkg/commands/git_commands"
@@ -28,11 +30,11 @@ import (
 	"github.com/jesseduffield/lazygit/pkg/gui/presentation/graph"
 	"github.com/jesseduffield/lazygit/pkg/gui/presentation/icons"
 	"github.com/jesseduffield/lazygit/pkg/gui/services/custom_commands"
+	"github.com/jesseduffield/lazygit/pkg/gui/status"
 	"github.com/jesseduffield/lazygit/pkg/gui/style"
 	"github.com/jesseduffield/lazygit/pkg/gui/types"
 	"github.com/jesseduffield/lazygit/pkg/integration/components"
 	integrationTypes "github.com/jesseduffield/lazygit/pkg/integration/types"
-	"github.com/jesseduffield/lazygit/pkg/snake"
 	"github.com/jesseduffield/lazygit/pkg/tasks"
 	"github.com/jesseduffield/lazygit/pkg/theme"
 	"github.com/jesseduffield/lazygit/pkg/updates"
@@ -41,35 +43,11 @@ import (
 	"gopkg.in/ozeidan/fuzzy-patricia.v3/patricia"
 )
 
-// screen sizing determines how much space your selected window takes up (window
-// as in panel, not your terminal's window). Sometimes you want a bit more space
-// to see the contents of a panel, and this keeps track of how much maximisation
-// you've set
-type WindowMaximisation int
-
-const (
-	SCREEN_NORMAL WindowMaximisation = iota
-	SCREEN_HALF
-	SCREEN_FULL
-)
-
 const StartupPopupVersion = 5
 
 // OverlappingEdges determines if panel edges overlap
 var OverlappingEdges = false
 
-type ContextManager struct {
-	ContextStack []types.Context
-	sync.RWMutex
-}
-
-func NewContextManager() ContextManager {
-	return ContextManager{
-		ContextStack: []types.Context{},
-		RWMutex:      sync.RWMutex{},
-	}
-}
-
 type Repo string
 
 // Gui wraps the gocui Gui object which handles rendering and events
@@ -90,7 +68,7 @@ type Gui struct {
 	RepoStateMap         map[Repo]*GuiRepoState
 	Config               config.AppConfigurer
 	Updater              *updates.Updater
-	statusManager        *statusManager
+	statusManager        *status.StatusManager
 	waitForIntro         sync.WaitGroup
 	fileWatcher          *fileWatcher
 	viewBufferManagerMap map[string]*tasks.ViewBufferManager
@@ -106,10 +84,6 @@ type Gui struct {
 
 	Mutexes types.Mutexes
 
-	// findSuggestions will take a string that the user has typed into a prompt
-	// and return a slice of suggestions which match that string.
-	findSuggestions func(string) []*types.Suggestion
-
 	// when you enter into a submodule we'll append the superproject's path to this array
 	// so that you can return to the superproject
 	RepoPathStack *utils.StringStack
@@ -117,21 +91,14 @@ type Gui struct {
 	// this tells us whether our views have been initially set up
 	ViewsSetup bool
 
-	Views Views
+	Views types.Views
 
-	// if we've suspended the gui (e.g. because we've switched to a subprocess)
-	// we typically want to pause some things that are running like background
-	// file refreshes
-	PauseBackgroundThreads bool
-
-	// Log of the commands that get run, to be displayed to the user.
-	CmdLog []string
+	// Log of the commands/actions logged in the Command Log panel.
+	GuiLog []string
 
 	// the extras window contains things like the command log
 	ShowExtrasWindow bool
 
-	suggestionsAsyncHandler *tasks.AsyncHandler
-
 	PopupHandler types.IPopupHandler
 
 	IsNewRepo bool
@@ -139,6 +106,8 @@ type Gui struct {
 	// flag as to whether or not the diff view should ignore whitespace
 	IgnoreWhitespaceInDiffView bool
 
+	IsRefreshingFiles bool
+
 	// we use this to decide whether we'll return to the original directory that
 	// lazygit was opened in, or if we'll retain the one we're currently in.
 	RetainOriginalDir bool
@@ -153,10 +122,72 @@ type Gui struct {
 	// process
 	InitialDir string
 
-	c       *types.HelperCommon
+	BackgroundRoutineMgr *BackgroundRoutineMgr
+	// for accessing the gui's state from outside this package
+	stateAccessor *StateAccessor
+
+	Updating bool
+
+	c       *helpers.HelperCommon
 	helpers *helpers.Helpers
 
-	snakeGame *snake.Game
+	integrationTest integrationTypes.IntegrationTest
+
+	afterLayoutFuncs chan func() error
+}
+
+type StateAccessor struct {
+	gui *Gui
+}
+
+var _ types.IStateAccessor = new(StateAccessor)
+
+func (self *StateAccessor) GetIgnoreWhitespaceInDiffView() bool {
+	return self.gui.IgnoreWhitespaceInDiffView
+}
+
+func (self *StateAccessor) SetIgnoreWhitespaceInDiffView(value bool) {
+	self.gui.IgnoreWhitespaceInDiffView = value
+}
+
+func (self *StateAccessor) GetRepoPathStack() *utils.StringStack {
+	return self.gui.RepoPathStack
+}
+
+func (self *StateAccessor) GetUpdating() bool {
+	return self.gui.Updating
+}
+
+func (self *StateAccessor) SetUpdating(value bool) {
+	self.gui.Updating = value
+}
+
+func (self *StateAccessor) GetRepoState() types.IRepoStateAccessor {
+	return self.gui.State
+}
+
+func (self *StateAccessor) GetIsRefreshingFiles() bool {
+	return self.gui.IsRefreshingFiles
+}
+
+func (self *StateAccessor) SetIsRefreshingFiles(value bool) {
+	self.gui.IsRefreshingFiles = value
+}
+
+func (self *StateAccessor) GetShowExtrasWindow() bool {
+	return self.gui.ShowExtrasWindow
+}
+
+func (self *StateAccessor) SetShowExtrasWindow(value bool) {
+	self.gui.ShowExtrasWindow = value
+}
+
+func (self *StateAccessor) GetRetainOriginalDir() bool {
+	return self.gui.RetainOriginalDir
+}
+
+func (self *StateAccessor) SetRetainOriginalDir(value bool) {
+	self.gui.RetainOriginalDir = value
 }
 
 // we keep track of some stuff from one render to the next to see if certain
@@ -171,19 +202,14 @@ type GuiRepoState struct {
 	Model *types.Model
 	Modes *types.Modes
 
-	// Suggestions will sometimes appear when typing into a prompt
-	Suggestions []*types.Suggestion
-
-	Updating       bool
 	SplitMainPanel bool
 	LimitCommits   bool
 
-	IsRefreshingFiles bool
-	Searching         searchingState
-	StartupStage      StartupStage // Allows us to not load everything at once
+	SearchState  *types.SearchState
+	StartupStage types.StartupStage // Allows us to not load everything at once
 
-	ContextManager ContextManager
-	Contexts       *context.ContextTree
+	ContextMgr *ContextMgr
+	Contexts   *context.ContextTree
 
 	// WindowViewNameMap is a mapping of windows to the current view of that window.
 	// Some views move between windows for example the commitFiles view and when cycling through
@@ -195,24 +221,60 @@ type GuiRepoState struct {
 	// back in sync with the repo state
 	ViewsSetup bool
 
-	ScreenMode WindowMaximisation
+	ScreenMode types.WindowMaximisation
 
 	CurrentPopupOpts *types.CreatePopupPanelOpts
 }
 
-type searchingState struct {
-	view         *gocui.View
-	isSearching  bool
-	searchString string
+var _ types.IRepoStateAccessor = new(GuiRepoState)
+
+func (self *GuiRepoState) GetViewsSetup() bool {
+	return self.ViewsSetup
 }
 
-// startup stages so we don't need to load everything at once
-type StartupStage int
+func (self *GuiRepoState) GetWindowViewNameMap() *utils.ThreadSafeMap[string, string] {
+	return self.WindowViewNameMap
+}
 
-const (
-	INITIAL StartupStage = iota
-	COMPLETE
-)
+func (self *GuiRepoState) GetStartupStage() types.StartupStage {
+	return self.StartupStage
+}
+
+func (self *GuiRepoState) SetStartupStage(value types.StartupStage) {
+	self.StartupStage = value
+}
+
+func (self *GuiRepoState) GetCurrentPopupOpts() *types.CreatePopupPanelOpts {
+	return self.CurrentPopupOpts
+}
+
+func (self *GuiRepoState) SetCurrentPopupOpts(value *types.CreatePopupPanelOpts) {
+	self.CurrentPopupOpts = value
+}
+
+func (self *GuiRepoState) GetScreenMode() types.WindowMaximisation {
+	return self.ScreenMode
+}
+
+func (self *GuiRepoState) SetScreenMode(value types.WindowMaximisation) {
+	self.ScreenMode = value
+}
+
+func (self *GuiRepoState) InSearchPrompt() bool {
+	return self.SearchState.SearchType() != types.SearchTypeNone
+}
+
+func (self *GuiRepoState) GetSearchState() *types.SearchState {
+	return self.SearchState
+}
+
+func (self *GuiRepoState) SetSplitMainPanel(value bool) {
+	self.SplitMainPanel = value
+}
+
+func (self *GuiRepoState) GetSplitMainPanel() bool {
+	return self.SplitMainPanel
+}
 
 func (gui *Gui) onNewRepo(startArgs appTypes.StartArgs, reuseState bool) error {
 	var err error
@@ -227,14 +289,18 @@ func (gui *Gui) onNewRepo(startArgs appTypes.StartArgs, reuseState bool) error {
 		return err
 	}
 
-	gui.resetState(startArgs, reuseState)
+	contextToPush := gui.resetState(startArgs, reuseState)
 
-	gui.resetControllers()
+	gui.resetHelpersAndControllers()
 
 	if err := gui.resetKeybindings(); err != nil {
 		return err
 	}
 
+	if err := gui.c.PushContext(contextToPush); err != nil {
+		return err
+	}
+
 	return nil
 }
 
@@ -247,7 +313,7 @@ func (gui *Gui) onNewRepo(startArgs appTypes.StartArgs, reuseState bool) error {
 // it gets a bit confusing to land back in the status panel when visiting a repo
 // you've already switched from. There's no doubt some easy way to make the UX
 // optimal for all cases but I'm too lazy to think about what that is right now
-func (gui *Gui) resetState(startArgs appTypes.StartArgs, reuseState bool) {
+func (gui *Gui) resetState(startArgs appTypes.StartArgs, reuseState bool) types.Context {
 	currentDir, err := os.Getwd()
 
 	if reuseState {
@@ -262,7 +328,7 @@ func (gui *Gui) resetState(startArgs appTypes.StartArgs, reuseState bool) {
 				gui.State.CurrentPopupOpts = nil
 				gui.Mutexes.PopupMutex.Unlock()
 
-				return
+				return gui.c.CurrentContext()
 			}
 		} else {
 			gui.c.Log.Error(err)
@@ -271,11 +337,8 @@ func (gui *Gui) resetState(startArgs appTypes.StartArgs, reuseState bool) {
 
 	contextTree := gui.contextTree()
 
-	initialContext := initialContext(contextTree, startArgs)
 	initialScreenMode := initialScreenMode(startArgs, gui.Config)
 
-	initialWindowViewNameMap := gui.initialWindowViewNameMap(contextTree)
-
 	gui.State = &GuiRepoState{
 		Model: &types.Model{
 			CommitFiles:           nil,
@@ -293,32 +356,41 @@ func (gui *Gui) resetState(startArgs appTypes.StartArgs, reuseState bool) {
 			Diffing:       diffing.New(),
 		},
 		ScreenMode: initialScreenMode,
-		// TODO: put contexts in the context manager
-		ContextManager:    NewContextManager(),
+		// TODO: only use contexts from context manager
+		ContextMgr:        NewContextMgr(gui, contextTree),
 		Contexts:          contextTree,
-		WindowViewNameMap: initialWindowViewNameMap,
-	}
-
-	if err := gui.c.PushContext(initialContext); err != nil {
-		gui.c.Log.Error(err)
+		WindowViewNameMap: initialWindowViewNameMap(contextTree),
+		SearchState:       types.NewSearchState(),
 	}
 
 	gui.RepoStateMap[Repo(currentDir)] = gui.State
+
+	return initialContext(contextTree, startArgs)
 }
 
-func initialScreenMode(startArgs appTypes.StartArgs, config config.AppConfigurer) WindowMaximisation {
+func initialWindowViewNameMap(contextTree *context.ContextTree) *utils.ThreadSafeMap[string, string] {
+	result := utils.NewThreadSafeMap[string, string]()
+
+	for _, context := range contextTree.Flatten() {
+		result.Set(context.GetWindowName(), context.GetViewName())
+	}
+
+	return result
+}
+
+func initialScreenMode(startArgs appTypes.StartArgs, config config.AppConfigurer) types.WindowMaximisation {
 	if startArgs.FilterPath != "" || startArgs.GitArg != appTypes.GitArgNone {
-		return SCREEN_HALF
+		return types.SCREEN_HALF
 	} else {
 		defaultWindowSize := config.GetUserConfig().Gui.WindowSize
 
 		switch defaultWindowSize {
 		case "half":
-			return SCREEN_HALF
+			return types.SCREEN_HALF
 		case "full":
-			return SCREEN_FULL
+			return types.SCREEN_FULL
 		default:
-			return SCREEN_NORMAL
+			return types.SCREEN_NORMAL
 		}
 	}
 }
@@ -346,6 +418,10 @@ func initialContext(contextTree *context.ContextTree, startArgs appTypes.StartAr
 	return initialContext
 }
 
+func (gui *Gui) Contexts() *context.ContextTree {
+	return gui.State.Contexts
+}
+
 // for now the split view will always be on
 // NewGui builds a new gui handler
 func NewGui(
@@ -357,52 +433,56 @@ func NewGui(
 	initialDir string,
 ) (*Gui, error) {
 	gui := &Gui{
-		Common:                  cmn,
-		gitVersion:              gitVersion,
-		Config:                  config,
-		Updater:                 updater,
-		statusManager:           &statusManager{},
-		viewBufferManagerMap:    map[string]*tasks.ViewBufferManager{},
-		viewPtmxMap:             map[string]*os.File{},
-		showRecentRepos:         showRecentRepos,
-		RepoPathStack:           &utils.StringStack{},
-		RepoStateMap:            map[Repo]*GuiRepoState{},
-		CmdLog:                  []string{},
-		suggestionsAsyncHandler: tasks.NewAsyncHandler(),
+		Common:               cmn,
+		gitVersion:           gitVersion,
+		Config:               config,
+		Updater:              updater,
+		statusManager:        status.NewStatusManager(),
+		viewBufferManagerMap: map[string]*tasks.ViewBufferManager{},
+		viewPtmxMap:          map[string]*os.File{},
+		showRecentRepos:      showRecentRepos,
+		RepoPathStack:        &utils.StringStack{},
+		RepoStateMap:         map[Repo]*GuiRepoState{},
+		GuiLog:               []string{},
 
 		// originally we could only hide the command log permanently via the config
 		// but now we do it via state. So we need to still support the config for the
 		// sake of backwards compatibility. We're making use of short circuiting here
 		ShowExtrasWindow: cmn.UserConfig.Gui.ShowCommandLog && !config.GetAppState().HideCommandLog,
 		Mutexes: types.Mutexes{
-			RefreshingFilesMutex:  &deadlock.Mutex{},
-			RefreshingStatusMutex: &deadlock.Mutex{},
-			SyncMutex:             &deadlock.Mutex{},
-			LocalCommitsMutex:     &deadlock.Mutex{},
-			SubCommitsMutex:       &deadlock.Mutex{},
-			SubprocessMutex:       &deadlock.Mutex{},
-			PopupMutex:            &deadlock.Mutex{},
-			PtyMutex:              &deadlock.Mutex{},
+			RefreshingFilesMutex:    &deadlock.Mutex{},
+			RefreshingBranchesMutex: &deadlock.Mutex{},
+			RefreshingStatusMutex:   &deadlock.Mutex{},
+			SyncMutex:               &deadlock.Mutex{},
+			LocalCommitsMutex:       &deadlock.Mutex{},
+			SubCommitsMutex:         &deadlock.Mutex{},
+			SubprocessMutex:         &deadlock.Mutex{},
+			PopupMutex:              &deadlock.Mutex{},
+			PtyMutex:                &deadlock.Mutex{},
 		},
-		InitialDir: initialDir,
+		InitialDir:       initialDir,
+		afterLayoutFuncs: make(chan func() error, 1000),
 	}
 
-	gui.watchFilesForChanges()
+	gui.WatchFilesForChanges()
 
 	gui.PopupHandler = popup.NewPopupHandler(
 		cmn,
-		gui.createPopupPanel,
+		func(ctx goContext.Context, opts types.CreatePopupPanelOpts) error {
+			return gui.helpers.Confirmation.CreatePopupPanel(ctx, opts)
+		},
 		func() error { return gui.c.Refresh(types.RefreshOptions{Mode: types.ASYNC}) },
-		gui.popContext,
-		gui.currentContext,
+		func() error { return gui.State.ContextMgr.Pop() },
+		func() types.Context { return gui.State.ContextMgr.Current() },
 		gui.createMenu,
-		gui.withWaitingStatus,
-		gui.toast,
+		func(message string, f func(gocui.Task) error) { gui.helpers.AppStatus.WithWaitingStatus(message, f) },
+		func(message string) { gui.helpers.AppStatus.Toast(message) },
 		func() string { return gui.Views.Confirmation.TextArea.GetContent() },
+		func(f func(gocui.Task)) { gui.c.OnWorker(f) },
 	)
 
 	guiCommon := &guiCommon{gui: gui, IPopupHandler: gui.PopupHandler}
-	helperCommon := &types.HelperCommon{IGuiCommon: guiCommon, Common: cmn}
+	helperCommon := &helpers.HelperCommon{IGuiCommon: guiCommon, Common: cmn, IGetContexts: gui}
 
 	credentialsHelper := helpers.NewCredentialsHelper(helperCommon)
 
@@ -422,9 +502,16 @@ func NewGui(
 	gui.c = helperCommon
 
 	authors.SetCustomAuthors(gui.UserConfig.Gui.AuthorColors)
-	icons.SetIconEnabled(gui.UserConfig.Gui.ShowIcons)
+	if gui.UserConfig.Gui.NerdFontsVersion != "" {
+		icons.SetNerdFontsVersion(gui.UserConfig.Gui.NerdFontsVersion)
+	} else if gui.UserConfig.Gui.ShowIcons {
+		icons.SetNerdFontsVersion("2")
+	}
 	presentation.SetCustomBranches(gui.UserConfig.Gui.BranchColors)
 
+	gui.BackgroundRoutineMgr = &BackgroundRoutineMgr{gui: gui}
+	gui.stateAccessor = &StateAccessor{gui: gui}
+
 	return gui, nil
 }
 
@@ -435,9 +522,29 @@ var RuneReplacements = map[rune]string{
 }
 
 func (gui *Gui) initGocui(headless bool, test integrationTypes.IntegrationTest) (*gocui.Gui, error) {
-	playRecording := test != nil && os.Getenv(components.SANDBOX_ENV_VAR) != "true"
+	runInSandbox := os.Getenv(components.SANDBOX_ENV_VAR) == "true"
+	playRecording := test != nil && !runInSandbox
 
-	g, err := gocui.NewGui(gocui.OutputTrue, OverlappingEdges, playRecording, headless, RuneReplacements)
+	width, height := 0, 0
+	if test != nil {
+		if test.RequiresHeadless() {
+			if runInSandbox {
+				panic("Test requires headless, can't run in sandbox")
+			}
+			headless = true
+		}
+		width, height = test.HeadlessDimensions()
+	}
+
+	g, err := gocui.NewGui(gocui.NewGuiOpts{
+		OutputMode:       gocui.OutputTrue,
+		SupportOverlaps:  OverlappingEdges,
+		PlayRecording:    playRecording,
+		Headless:         headless,
+		RuneReplacements: RuneReplacements,
+		Width:            width,
+		Height:           height,
+	})
 	if err != nil {
 		return nil, err
 	}
@@ -503,11 +610,12 @@ func (gui *Gui) Run(startArgs appTypes.StartArgs) error {
 	})
 	deadlock.Opts.Disable = !gui.Debug
 
-	gui.g.OnSearchEscape = gui.onSearchEscape
 	if err := gui.Config.ReloadUserConfig(); err != nil {
 		return nil
 	}
 	userConfig := gui.UserConfig
+
+	gui.g.OnSearchEscape = func() error { gui.helpers.Search.Cancel(); return nil }
 	gui.g.SearchEscapeKey = keybindings.GetKey(userConfig.Keybinding.Universal.Return)
 	gui.g.NextSearchMatchKey = keybindings.GetKey(userConfig.Keybinding.Universal.NextMatch)
 	gui.g.PrevSearchMatchKey = keybindings.GetKey(userConfig.Keybinding.Universal.PrevMatch)
@@ -535,11 +643,12 @@ func (gui *Gui) Run(startArgs appTypes.StartArgs) error {
 
 	gui.waitForIntro.Add(1)
 
-	gui.startBackgroundRoutines()
+	gui.BackgroundRoutineMgr.startBackgroundRoutines()
 
 	gui.c.Log.Info("starting main loop")
 
-	gui.handleTestMode(startArgs.IntegrationTest)
+	// setting here so we can use it in layout.go
+	gui.integrationTest = startArgs.IntegrationTest
 
 	return gui.g.MainLoop()
 }
@@ -560,12 +669,12 @@ func (gui *Gui) RunAndHandleError(startArgs appTypes.StartArgs) error {
 
 			switch err {
 			case gocui.ErrQuit:
-				if gui.RetainOriginalDir {
-					if err := gui.recordDirectory(gui.InitialDir); err != nil {
+				if gui.c.State().GetRetainOriginalDir() {
+					if err := gui.helpers.RecordDirectory.RecordDirectory(gui.InitialDir); err != nil {
 						return err
 					}
 				} else {
-					if err := gui.recordCurrentDirectory(); err != nil {
+					if err := gui.helpers.RecordDirectory.RecordCurrentDirectory(); err != nil {
 						return err
 					}
 				}
@@ -635,7 +744,8 @@ func (gui *Gui) runSubprocessWithSuspense(subprocess oscommands.ICmdObj) (bool,
 		return false, gui.c.Error(err)
 	}
 
-	gui.PauseBackgroundThreads = true
+	gui.BackgroundRoutineMgr.PauseBackgroundRefreshes(true)
+	defer gui.BackgroundRoutineMgr.PauseBackgroundRefreshes(false)
 
 	cmdErr := gui.runSubprocess(subprocess)
 
@@ -643,8 +753,6 @@ func (gui *Gui) runSubprocessWithSuspense(subprocess oscommands.ICmdObj) (bool,
 		return false, err
 	}
 
-	gui.PauseBackgroundThreads = false
-
 	if cmdErr != nil {
 		return false, gui.c.Error(cmdErr)
 	}
@@ -695,37 +803,23 @@ func (gui *Gui) loadNewRepo() error {
 	return nil
 }
 
-func (gui *Gui) showInitialPopups(tasks []func(chan struct{}) error) {
-	gui.waitForIntro.Add(len(tasks))
-	done := make(chan struct{})
+func (gui *Gui) showIntroPopupMessage() {
+	gui.waitForIntro.Add(1)
 
-	go utils.Safe(func() {
-		for _, task := range tasks {
-			task := task
-			go utils.Safe(func() {
-				if err := task(done); err != nil {
-					_ = gui.c.Error(err)
-				}
-			})
-
-			<-done
+	gui.c.OnUIThread(func() error {
+		onConfirm := func() error {
+			gui.c.GetAppState().StartupPopupVersion = StartupPopupVersion
+			err := gui.c.SaveAppState()
 			gui.waitForIntro.Done()
+			return err
 		}
-	})
-}
 
-func (gui *Gui) showIntroPopupMessage(done chan struct{}) error {
-	onConfirm := func() error {
-		done <- struct{}{}
-		gui.c.GetAppState().StartupPopupVersion = StartupPopupVersion
-		return gui.c.SaveAppState()
-	}
-
-	return gui.c.Confirm(types.ConfirmOpts{
-		Title:         "",
-		Prompt:        gui.c.Tr.IntroPopupMessage,
-		HandleConfirm: onConfirm,
-		HandleClose:   onConfirm,
+		return gui.c.Confirm(types.ConfirmOpts{
+			Title:         "",
+			Prompt:        gui.c.Tr.IntroPopupMessage,
+			HandleConfirm: onConfirm,
+			HandleClose:   onConfirm,
+		})
 	})
 }
 
@@ -747,3 +841,11 @@ func (gui *Gui) onUIThread(f func() error) {
 		return f()
 	})
 }
+
+func (gui *Gui) onWorker(f func(gocui.Task)) {
+	gui.g.OnWorker(f)
+}
+
+func (gui *Gui) getWindowDimensions(informationStr string, appStatus string) map[string]boxlayout.Dimensions {
+	return gui.helpers.WindowArrangement.GetWindowDimensions(informationStr, appStatus)
+}
