@@ -4,11 +4,10 @@ import (
 	"github.com/jesseduffield/generics/slices"
 	"github.com/jesseduffield/gocui"
 	"github.com/jesseduffield/lazygit/pkg/gui/context"
+	"github.com/jesseduffield/lazygit/pkg/gui/types"
 	"github.com/jesseduffield/lazygit/pkg/theme"
 )
 
-const SEARCH_PREFIX = "search: "
-
 // layout is called for every screen re-render e.g. when the screen is resized
 func (gui *Gui) layout(g *gocui.Gui) error {
 	if !gui.ViewsSetup {
@@ -23,7 +22,8 @@ func (gui *Gui) layout(g *gocui.Gui) error {
 	width, height := g.Size()
 
 	informationStr := gui.informationStr()
-	appStatus := gui.statusManager.getStatusString()
+
+	appStatus := gui.helpers.AppStatus.GetStatusString()
 
 	viewDimensions := gui.getWindowDimensions(informationStr, appStatus)
 
@@ -96,16 +96,16 @@ func (gui *Gui) layout(g *gocui.Gui) error {
 
 	gui.Views.Tooltip.Visible = gui.Views.Menu.Visible && gui.Views.Tooltip.Buffer() != ""
 
-	for _, context := range gui.TransientContexts() {
+	for _, context := range gui.transientContexts() {
 		view, err := gui.g.View(context.GetViewName())
 		if err != nil && !gocui.IsUnknownView(err) {
 			return err
 		}
-		view.Visible = gui.getViewNameForWindow(context.GetWindowName()) == context.GetViewName()
+		view.Visible = gui.helpers.Window.GetViewNameForWindow(context.GetWindowName()) == context.GetViewName()
 	}
 
 	if gui.PrevLayout.Information != informationStr {
-		gui.setViewContent(gui.Views.Information, informationStr)
+		gui.c.SetViewContent(gui.Views.Information, informationStr)
 		gui.PrevLayout.Information = informationStr
 	}
 
@@ -114,6 +114,8 @@ func (gui *Gui) layout(g *gocui.Gui) error {
 			return err
 		}
 
+		gui.handleTestMode()
+
 		gui.ViewsSetup = true
 	}
 
@@ -125,29 +127,13 @@ func (gui *Gui) layout(g *gocui.Gui) error {
 		gui.State.ViewsSetup = true
 	}
 
-	for _, listContext := range gui.getListContexts() {
+	for _, listContext := range gui.c.Context().AllList() {
 		view, err := gui.g.View(listContext.GetViewName())
 		if err != nil {
 			continue
 		}
 
-		listContext.FocusLine()
-
 		view.SelBgColor = theme.GocuiSelectedLineBgColor
-
-		// I doubt this is expensive though it's admittedly redundant after the first render
-		view.SetOnSelectItem(gui.onSelectItemWrapper(listContext.OnSearchSelect))
-	}
-
-	for _, context := range gui.getPatchExplorerContexts() {
-		context := context
-		context.GetView().SetOnSelectItem(gui.onSelectItemWrapper(
-			func(selectedLineIdx int) error {
-				context.GetMutex().Lock()
-				defer context.GetMutex().Unlock()
-				return context.NavigateTo(gui.c.IsCurrentContext(context), selectedLineIdx)
-			}),
-		)
 	}
 
 	mainViewWidth, mainViewHeight := gui.Views.Main.Size()
@@ -163,7 +149,23 @@ func (gui *Gui) layout(g *gocui.Gui) error {
 	// if you run `lazygit --logs`
 	// this will let you see these branches as prettified json
 	// gui.c.Log.Info(utils.AsJson(gui.State.Model.Branches[0:4]))
-	return gui.resizeCurrentPopupPanel()
+	if err := gui.helpers.Confirmation.ResizeCurrentPopupPanel(); err != nil {
+		return err
+	}
+
+outer:
+	for {
+		select {
+		case f := <-gui.afterLayoutFuncs:
+			if err := f(); err != nil {
+				return err
+			}
+		default:
+			break outer
+		}
+	}
+
+	return nil
 }
 
 func (gui *Gui) prepareView(viewName string) (*gocui.View, error) {
@@ -181,7 +183,7 @@ func (gui *Gui) onInitialViewsCreationForRepo() error {
 		}
 	}
 
-	initialContext := gui.currentSideContext()
+	initialContext := gui.c.CurrentContext()
 	if err := gui.c.ActivateContext(initialContext); err != nil {
 		return err
 	}
@@ -189,6 +191,16 @@ func (gui *Gui) onInitialViewsCreationForRepo() error {
 	return gui.loadNewRepo()
 }
 
+func (gui *Gui) popupViewNames() []string {
+	popups := slices.Filter(gui.State.Contexts.Flatten(), func(c types.Context) bool {
+		return c.GetKind() == types.PERSISTENT_POPUP || c.GetKind() == types.TEMPORARY_POPUP
+	})
+
+	return slices.Map(popups, func(c types.Context) string {
+		return c.GetViewName()
+	})
+}
+
 func (gui *Gui) onInitialViewsCreation() error {
 	// now we order the views (in order of bottom first)
 	for _, view := range gui.orderedViews() {
@@ -217,24 +229,57 @@ func (gui *Gui) onInitialViewsCreation() error {
 	gui.g.Mutexes.ViewsMutex.Unlock()
 
 	if !gui.c.UserConfig.DisableStartupPopups {
-		popupTasks := []func(chan struct{}) error{}
 		storedPopupVersion := gui.c.GetAppState().StartupPopupVersion
 		if storedPopupVersion < StartupPopupVersion {
-			popupTasks = append(popupTasks, gui.showIntroPopupMessage)
+			gui.showIntroPopupMessage()
 		}
-		gui.showInitialPopups(popupTasks)
 	}
 
 	if gui.showRecentRepos {
-		if err := gui.handleCreateRecentReposMenu(); err != nil {
+		if err := gui.helpers.Repos.CreateRecentReposMenu(); err != nil {
 			return err
 		}
 		gui.showRecentRepos = false
 	}
 
-	gui.Updater.CheckForNewUpdate(gui.onBackgroundUpdateCheckFinish, false)
+	gui.helpers.Update.CheckForUpdateInBackground()
 
 	gui.waitForIntro.Done()
 
 	return nil
 }
+
+// getFocusLayout returns a manager function for when view gain and lose focus
+func (gui *Gui) getFocusLayout() func(g *gocui.Gui) error {
+	var previousView *gocui.View
+	return func(g *gocui.Gui) error {
+		newView := gui.g.CurrentView()
+		// for now we don't consider losing focus to a popup panel as actually losing focus
+		if newView != previousView && !gui.helpers.Confirmation.IsPopupPanel(newView.Name()) {
+			if err := gui.onViewFocusLost(previousView); err != nil {
+				return err
+			}
+
+			previousView = newView
+		}
+		return nil
+	}
+}
+
+func (gui *Gui) onViewFocusLost(oldView *gocui.View) error {
+	if oldView == nil {
+		return nil
+	}
+
+	oldView.Highlight = false
+
+	_ = oldView.SetOriginX(0)
+
+	return nil
+}
+
+func (gui *Gui) transientContexts() []types.Context {
+	return slices.Filter(gui.State.Contexts.Flatten(), func(context types.Context) bool {
+		return context.IsTransient()
+	})
+}
