@@ -1,6 +1,8 @@
 package presentation
 
 import (
+	"time"
+
 	"github.com/jesseduffield/generics/set"
 	"github.com/jesseduffield/generics/slices"
 	"github.com/jesseduffield/lazygit/pkg/commands/models"
@@ -10,7 +12,7 @@ import (
 	"github.com/kyokomi/emoji/v2"
 )
 
-func GetReflogCommitListDisplayStrings(commits []*models.Commit, fullDescription bool, cherryPickedCommitShaSet *set.Set[string], diffName string, timeFormat string, parseEmoji bool) [][]string {
+func GetReflogCommitListDisplayStrings(commits []*models.Commit, fullDescription bool, cherryPickedCommitShaSet *set.Set[string], diffName string, now time.Time, timeFormat string, shortTimeFormat string, parseEmoji bool) [][]string {
 	var displayFunc func(*models.Commit, reflogCommitDisplayAttributes) []string
 	if fullDescription {
 		displayFunc = getFullDescriptionDisplayStringsForReflogCommit
@@ -23,10 +25,12 @@ func GetReflogCommitListDisplayStrings(commits []*models.Commit, fullDescription
 		cherryPicked := cherryPickedCommitShaSet.Includes(commit.Sha)
 		return displayFunc(commit,
 			reflogCommitDisplayAttributes{
-				cherryPicked: cherryPicked,
-				diffed:       diffed,
-				parseEmoji:   parseEmoji,
-				timeFormat:   timeFormat,
+				cherryPicked:    cherryPicked,
+				diffed:          diffed,
+				parseEmoji:      parseEmoji,
+				timeFormat:      timeFormat,
+				shortTimeFormat: shortTimeFormat,
+				now:             now,
 			})
 	})
 }
@@ -45,10 +49,12 @@ func reflogShaColor(cherryPicked, diffed bool) style.TextStyle {
 }
 
 type reflogCommitDisplayAttributes struct {
-	cherryPicked bool
-	diffed       bool
-	parseEmoji   bool
-	timeFormat   string
+	cherryPicked    bool
+	diffed          bool
+	parseEmoji      bool
+	timeFormat      string
+	shortTimeFormat string
+	now             time.Time
 }
 
 func getFullDescriptionDisplayStringsForReflogCommit(c *models.Commit, attrs reflogCommitDisplayAttributes) []string {
@@ -59,7 +65,7 @@ func getFullDescriptionDisplayStringsForReflogCommit(c *models.Commit, attrs ref
 
 	return []string{
 		reflogShaColor(attrs.cherryPicked, attrs.diffed).Sprint(c.ShortSha()),
-		style.FgMagenta.Sprint(utils.UnixToDate(c.UnixTimestamp, attrs.timeFormat)),
+		style.FgMagenta.Sprint(utils.UnixToDateSmart(attrs.now, c.UnixTimestamp, attrs.timeFormat, attrs.shortTimeFormat)),
 		theme.DefaultTextColor.Sprint(name),
 	}
 }
