@@ -28,7 +28,10 @@ func RunTUI() {
 	app := newApp(testDir)
 	app.loadTests()
 
-	g, err := gocui.NewGui(gocui.OutputTrue, false, false, false, gui.RuneReplacements)
+	g, err := gocui.NewGui(gocui.NewGuiOpts{
+		OutputMode:       gocui.OutputTrue,
+		RuneReplacements: gui.RuneReplacements,
+	})
 	if err != nil {
 		log.Panicln(err)
 	}
