@@ -7,7 +7,7 @@ import (
 
 var CreateTag = NewIntegrationTest(NewIntegrationTestArgs{
 	Description:  "Create a new tag on a commit",
-	ExtraCmdArgs: "",
+	ExtraCmdArgs: []string{},
 	Skip:         false,
 	SetupConfig:  func(config *config.AppConfig) {},
 	SetupRepo: func(shell *Shell) {
@@ -25,7 +25,7 @@ var CreateTag = NewIntegrationTest(NewIntegrationTestArgs{
 
 		t.ExpectPopup().Menu().
 			Title(Equals("Create tag")).
-			Select(Contains("lightweight")).
+			Select(Contains("Lightweight")).
 			Confirm()
 
 		t.ExpectPopup().Prompt().
